@@ -12,6 +12,8 @@
 #define HIGH_DIALOG_COMPAT	"XDIALOG_HIGH_DIALOG_COMPAT"
 #define FORCE_AUTOSIZE		"XDIALOG_FORCE_AUTOSIZE"
 #define INFOBOX_TIMEOUT		"XDIALOG_INFOBOX_TIMEOUT"
+                                /* Temporary pipe filename used for printing */
+#define TEMP_FILE               "/tmp/Xdialog.tmp"
 
 /* You may change these as well IOT change the Xdialog limitations... */
 
@@ -35,7 +37,7 @@
  */
 
 #ifndef FIXED_FONT
-#define FIXED_FONT "-*-*-medium-r-normal-*-*-*-*-*-m-70-iso8859-1"
+#define FIXED_FONT "-*-*-medium-r-normal-*-*-*-*-*-m-70-*-*"
 #endif
 
 #ifndef PRINTER_CMD
@@ -44,10 +46,6 @@
 #ifndef PRINTER_CMD_OPTION
 #define PRINTER_CMD_OPTION "-P"		/* The option to specify the prt queue */
 #endif
-#ifndef TEMP_FILE
-#define TEMP_FILE "/tmp/Xdialog.tmp" 	/* Temporary file used for printing */
-#endif
-
 #ifdef HAVE_CATGETS			/* We don't support catgets */
 #error Xdialog does not get catgets support, try: ./configure --enable-nls
 #endif
@@ -142,7 +140,9 @@ typedef struct	{
 	gint		justify;				/* Xdialog labels justification method */
 	gint		buttons_style;				/* Xdialog buttons style */
 	gint		interval;				/* Xdialog output result time interval */
+	gint		timeout;				/* Xdialog user input timeout (in seconds) */
 	gint		timer;					/* Xdialog timer routine */
+	gint		timer2;					/* Xdialog timer routine #2 (for user timeout) */
 	gint		passwd;					/* Password flags for text entries */
 	gint		tips;					/* Tips flag (0 or 1) for tips in menu/list */
 	gint		beep;					/* Beep flag */
@@ -225,6 +225,10 @@ void create_menubox(gchar *optarg, gchar *options[], gint list_size);
 
 void create_filesel(gchar *optarg, gboolean dsel_flag);
 
+void create_colorsel(gchar *optarg);
+
+void create_fontsel(gchar *optarg);
+
 void create_calendar(gchar *optarg, gint day, gint month, gint year);
 
-void create_timebox(gchar *optarg);
+void create_timebox(gchar *optarg, gint hours, gint minutes, gint seconds);
