@@ -29,6 +29,7 @@
 
 #include "interface.h"
 #include "support.h"
+#include "time.h"
 
 /* A structure used to pass Xdialog parameters. */
 Xdialog_data Xdialog;
@@ -41,7 +42,7 @@ gboolean dialog_compat = FALSE;
 #define HELP_TEXT1 \
 "Xdialog v"VERSION" by Thierry Godefroy <xdialog@free.fr> (v1.0 was\n\
 written by Alfred at Cyberone Internet <alfred@cyberone.com.au>).\n\
-Xdialog home page available at: http://xdialog.free.fr.\n\
+Xdialog home page available at: http://xdialog.dyns.net/\n\
 \n\
 Usage: "
 
@@ -70,6 +71,7 @@ Transient options:\n\
   --reverse\n\
   --keep-colors\n\
   --interval <timeout>\n\
+  --timeout <timeout> (in seconds)\n\
   --no-tags\n\
   --item-help (if used, the {...} parameters are needed in menus/lists widgets)\n\
   --default-item <tag>\n\
@@ -81,7 +83,7 @@ Transient options:\n\
   --wizard\n\
   --help <help>\n\
   --print <printer> (1)\n\
-  --check <label>\n\
+  --check <label> [<status>]\n\
   --ok-label <label>\n\
   --cancel-label <label>\n\
   --beep\n\
@@ -117,8 +119,10 @@ Box options:\n\
   --treeview	<text> <height> <width> <list height> <tag1> <item1> <status1> <item_depth1> {<help1>}...\n\
   --fselect	<file> <height> <width>\n\
   --dselect	<directory> <height> <width>\n\
-  --calendar	<text> <height> <width> <day> <month> <year>\n\
-  --timebox	<text> <height> <width>\n\
+  --colorsel	<text> <height> <width>\n\
+  --fontsel	<font name> <height> <width>\n\
+  --calendar	<text> <height> <width> [<day> <month> <year>]\n\
+  --timebox	<text> <height> <width> [<hours> <minutes> <seconds>]\n\
 \n\
 Special options:\n\
   --version		(prints version number to stderr and exits).\n\
@@ -131,11 +135,26 @@ of X) which will represent the size of the Xdialog window in pixels. Specifying\
 a size of 0 0 (or 0x0) will auto-size Xdialog, while a size of -1 -1 (or -1x-1)\n\
 will maximize it.\n\
 \n\
-(1) This Xdialog binary compiled with: "PRINTER_CMD" "PRINTER_CMD_OPTION"<printer> "TEMP_FILE"\n\
+(1) This Xdialog binary compiled with: "PRINTER_CMD" "PRINTER_CMD_OPTION"<printer>\n\
     as the print command. If <printer> is \"\" (an empty string), the "PRINTER_CMD_OPTION"\n\
     option is not used.\n\n"
 
-#define HELP_MSG_SIZE 3900
+#ifdef USE_SCANF
+
+#define HELP_TEXT3 \
+"WARNING: This binary of Xdialog was compiled with the --with-scanf-calls configure\n\
+option. The --infobox and --gauge widgets may therefore not work completely as\n\
+expected (lack of refresh of the GTK+ menu while scanning the input stream).\n\
+You may try to re-compile Xdialog without specifying this configure option (but\n\
+then, your system C library may lack the necessary functions).\n\n"
+
+#define HELP_MSG_SIZE 4500
+
+#else
+
+#define HELP_MSG_SIZE 4100
+
+#endif
 
 /* List of all recognized Xdialog options */
 enum {
@@ -167,6 +186,8 @@ enum {
 	B_TREEVIEW,
 	B_FSELECT,
 	B_DSELECT,
+	B_COLORSEL,
+	B_FONTSEL,
 	B_CALENDAR,
 	B_TIMEBOX,
 	/* Common options */
@@ -214,6 +235,7 @@ enum {
 	T_CANCELLABEL,
 	T_ICON,
 	T_INTERVAL,
+	T_TIMEOUT,
 	T_CHECK,
 	T_BEEP,
 	T_BEEPAFTER,
@@ -231,12 +253,18 @@ static void print_help_info(char *name, char *errmsg)
 {
 	gchar msg[HELP_MSG_SIZE];
 	gchar cmd[32];
+#ifdef USE_GTK2
+	GtkTextBuffer *text_buffer;
+#endif
 
 	strcpy(cmd, strlen(name) < 32 ? name : XDIALOG);
 
 	strcpysafe(msg, HELP_TEXT1, HELP_MSG_SIZE);
 	strcatsafe(msg, cmd, HELP_MSG_SIZE);
 	strcatsafe(msg, HELP_TEXT2, HELP_MSG_SIZE);
+#ifdef USE_SCANF
+	strcatsafe(msg, HELP_TEXT3, HELP_MSG_SIZE);
+#endif
 
 	fprintf(stderr, "%s: %s !\n", cmd, errmsg);
 	fprintf(stderr, msg);
@@ -259,8 +287,13 @@ static void print_help_info(char *name, char *errmsg)
 	Xdialog.size_in_pixels = FALSE;
 	get_maxsize(&Xdialog.xsize, &Xdialog.ysize);
 	create_textbox("", FALSE);
+#ifdef USE_GTK2
+	text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Xdialog.widget1));
+	gtk_text_buffer_insert_at_cursor(text_buffer, msg, strlen(msg));
+#else
 	gtk_text_insert(GTK_TEXT(Xdialog.widget1), NULL, NULL, NULL, msg,
 			strlen(msg));
+#endif
 	gtk_widget_show(Xdialog.window);
 	gtk_main();
 
@@ -285,6 +318,58 @@ static gboolean is_true(const char *var_name)
 #endif
 }
 
+/* An equivalent to the g_log_default_handler() function of glib, with
+ * exclusive use of stderr (nothing is ever printed on stdout), and with a
+ * possibility to disable completely any message output, via the
+ * XDIALOG_NO_GMSGS environment variable.
+ */
+static void g_log_xdialog_handler(const gchar *log_domain,
+				  GLogLevelFlags log_level,
+				  const gchar *message, gpointer unused_data)
+{
+	gchar *recurse, *fatal, type[10];
+
+	if (is_true("XDIALOG_NO_GMSGS"))
+		return;
+
+	recurse = (log_level & G_LOG_FLAG_RECURSION) != 0 ? "" : "(recursed) ";
+	fatal = (log_level & G_LOG_FLAG_FATAL) != 0 ? "\naborting..." : "";
+	log_level &= G_LOG_LEVEL_MASK;
+
+	if (!message)
+		message = "g_log_default_handler(): (NULL) message";
+
+	switch (log_level) {
+		case G_LOG_LEVEL_ERROR:
+			strcpy(type, "ERROR");
+			break;
+		case G_LOG_LEVEL_CRITICAL:
+			strcpy(type, "CRITICAL");
+			break;
+		case G_LOG_LEVEL_WARNING:
+			strcpy(type, "WARNING");
+			break;
+		case G_LOG_LEVEL_MESSAGE:
+			strcpy(type, "Message");
+			break;
+		case G_LOG_LEVEL_INFO:
+			strcpy(type, "INFO");
+			break;
+		case G_LOG_LEVEL_DEBUG:
+			strcpy(type, "DEBUG");
+			break;
+		default:
+			sprintf(type, "LOG 0x%2x", log_level);
+	}
+
+	if (log_domain) {
+		fprintf(stderr, "\n%s-", log_domain);
+	} else {
+		fprintf(stderr, "\n** ");
+	}
+	fprintf(stderr, "%s %s**: %s%s\n", type, recurse, message, fatal);
+}
+
 /* Xsize, Ysize, Xorg and Yorg parameters */
 
 #ifdef HAVE_STRSTR
@@ -407,6 +492,22 @@ static int get_list_size(int argc, char *argv[], int *optind, int parameters,
 	return list_size;
 }
 
+static int param_number(int argc, char *argv[], int *optind)
+{
+	int next_opt, i;
+
+	next_opt = argc;
+
+	for (i = *optind; i < argc-1; i++)
+		if (strstr(argv[i], "--") == argv[i]) {
+			next_opt = i;
+			break;
+		}
+
+	return (next_opt - *optind);
+
+}
+
 /* There is a problem with getopt_long_only that returns an optcode of 63 when
  * it reaches an unknown option !
  */
@@ -430,8 +531,11 @@ int main(int argc, char *argv[])
 	gint list_size = 0;
 	gint min = 0, max = 0, deflt =0;
 	gint day = 0, month = 0, year = 0;
-	gint beep_tmp;
+	gint hours = 0, minutes = 0, seconds = 0;
+	gint beep_tmp, timeout_tmp;
 	gboolean icon_tmp, check_tmp;
+	time_t curr_time;
+	struct tm *localdate;
 	static struct option long_options[] = {
 		/* Box options */
 		{"yesno",		1, 0, B_YESNO},
@@ -462,6 +566,8 @@ int main(int argc, char *argv[])
                 {"treeview",		1, 0, B_TREEVIEW},
                 {"fselect",		1, 0, B_FSELECT},
                 {"dselect",		1, 0, B_DSELECT},
+                {"colorsel",		1, 0, B_COLORSEL},
+                {"fontsel",		1, 0, B_FONTSEL},
                 {"calendar",		1, 0, B_CALENDAR},
                 {"timebox",		1, 0, B_TIMEBOX},
 		/* Common options */
@@ -510,6 +616,7 @@ int main(int argc, char *argv[])
                 {"cancel-label",	1, 0, T_CANCELLABEL},
                 {"icon",		1, 0, T_ICON},
                 {"interval",		1, 0, T_INTERVAL},
+                {"timeout",		1, 0, T_TIMEOUT},
                 {"check",		1, 0, T_CHECK},
                 {"beep",		0, 0, T_BEEP},
                 {"beep-after",		0, 0, T_BEEPAFTER},
@@ -576,6 +683,7 @@ int main(int argc, char *argv[])
 #if 0	/* Not needed because of the memset: listed here as a reminder only... */
 	Xdialog.passwd		= 0;			/* Don't use passwd input as default */
 	Xdialog.interval	= 0;			/* Don't report periodically as default */
+	Xdialog.timeout		= 0;			/* Don't use a timeout */
 	Xdialog.tips		= 0;			/* Don't use tips for items in lists */
 	Xdialog.beep		= 0;			/* Don't beep */
 	Xdialog.backtitle[0]	= 0;			/* Defaults to no backtitle */
@@ -592,6 +700,19 @@ int main(int argc, char *argv[])
 		fprintf(stderr,
 			"%s: GTK+ version too old, please upgrade !\n", argv[0]);
 	gtk_set_locale();
+
+	/* Set custom log handler routines, so that GTK, GDK and GLIB never
+         * print anything on stdout, but always use stderr instead.
+	 */
+	g_log_set_handler("Gdk", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+			  | G_LOG_FLAG_RECURSION, g_log_xdialog_handler, NULL);
+	g_log_set_handler("GLib", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+			  | G_LOG_FLAG_RECURSION, g_log_xdialog_handler, NULL);
+	g_log_set_handler("Gtk", G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+			  | G_LOG_FLAG_RECURSION, g_log_xdialog_handler, NULL);
+	g_log_set_handler(NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL
+			  | G_LOG_FLAG_RECURSION, g_log_xdialog_handler, NULL);
+
 	if (!gtk_init_check(&argc, &argv)) {
 		fprintf(stderr,
 			"%s: Error initializing the GUI...\n"
@@ -821,27 +942,65 @@ show_again:
 				create_filesel(optarg, TRUE);
 				win = TRUE;
 				break;
+			case B_COLORSEL:	/* colour selector */
+				get_box_size(argc, argv, &optind);
+				create_colorsel(optarg);
+				win = TRUE;
+				break;
+			case B_FONTSEL:		/* font selector */
+				get_box_size(argc, argv, &optind);
+				create_fontsel(optarg);
+				win = TRUE;
+				break;
 			case B_CALENDAR:	/* a calendar */
 				get_box_size(argc, argv, &optind);
-				if (argv[optind] == NULL ||
-				    argv[optind+1] == NULL ||
-				    argv[optind+2] == NULL)
-					print_help_info(argv[0], "missing calendar parameter");
-				else {
+				list_size = param_number(argc, argv, &optind);
+				if (list_size != 0) {
+					if (list_size != 3)
+						print_help_info(argv[0], "incorrect number of parameters");
 					day   = atoi(argv[optind++]);
 					month = atoi(argv[optind++]);
 					year  = atoi(argv[optind++]);
+				} else {
+					day   = 0;
+					month = 0;
+					year  = 0;
 				}
 				if (day < 0 || day > 31 ||
 				    month < 0 || month > 12 ||
 				    (year != 0 && year < 1970))
 					print_help_info(argv[0], "bad calendar date");
+				if ( day == 0 || month == 0 || year == 0 ) {
+					time(&curr_time);
+					localdate = localtime(&curr_time);
+					day = localdate->tm_mday;
+					month = localdate->tm_mon + 1;
+					year = localdate->tm_year + 1900;
+				}
 				create_calendar(optarg, day, month, year);
 				win = TRUE;
 				break;
 			case B_TIMEBOX:		/* a time box */
 				get_box_size(argc, argv, &optind);
-				create_timebox(optarg);
+				list_size = param_number(argc, argv, &optind);
+				if (list_size != 0) {
+					if (list_size != 3)
+						print_help_info(argv[0], "incorrect number of parameters");
+					hours   = atoi(argv[optind++]);
+					minutes = atoi(argv[optind++]);
+					seconds = atoi(argv[optind++]);
+                                }  else {
+					time(&curr_time);
+					localdate = localtime(&curr_time);
+					hours     = localdate->tm_hour;
+					minutes   = localdate->tm_min;
+					seconds   = localdate->tm_sec;
+				}
+				if (hours < 0 || hours > 23 ||
+				    minutes < 0 || minutes > 59 ||
+				    seconds < 0 || seconds > 59)
+					print_help_info(argv[0], "bad time argument");
+				create_timebox(optarg, hours, minutes, seconds);
 				win = TRUE;
 				break;
 		/* Common options */
@@ -991,11 +1150,29 @@ show_again:
 				if (optarg != NULL)
 					Xdialog.interval = atoi(optarg);
 				if (Xdialog.interval < 0)
-					print_help_info(argv[0], "bad interval number");
+					print_help_info(argv[0], "bad interval value");
+				break;
+			case T_TIMEOUT:		/* --timeout option */
+				Xdialog.timeout = -1;
+				if (optarg != NULL)
+					Xdialog.timeout = atoi(optarg);
+				if (Xdialog.timeout < 0)
+					print_help_info(argv[0], "bad timeout value");
 				break;
 			case T_CHECK:		/* --check option */
 				strcpysafe(Xdialog.check_label, optarg, MAX_LABEL_LENGTH);
 				Xdialog.check = TRUE;
+				list_size = param_number(argc, argv, &optind);
+				if (list_size > 1)
+					print_help_info(argv[0], "incorrect number of parameters");
+				if (list_size)
+#ifdef HAVE_STRCASECMP
+					Xdialog.checked = !strcasecmp(argv[optind++], "on");
+#else
+					Xdialog.checked = !strcmp(argv[optind], "on") ||
+				                          !strcmp(argv[optind], "On") ||
+				                          !strcmp(argv[optind++], "ON");
+#endif
 				break;
 			case T_BEEP:		/* --beep option */
 				Xdialog.beep |= BEEP_BEFORE;
@@ -1057,11 +1234,14 @@ show_again:
 					Xdialog.check = FALSE;
 					beep_tmp = Xdialog.beep;
 					Xdialog.beep = 0;
+					timeout_tmp = Xdialog.timeout;
+					Xdialog.timeout = 0;
 					create_msgbox(help_text, FALSE);
 					gtk_widget_show(Xdialog.window);
 					gtk_main();
 					if (Xdialog.exit_code != 0)
 						return Xdialog.exit_code;
+					Xdialog.timeout = timeout_tmp;
 					Xdialog.beep = beep_tmp;
 					Xdialog.icon = icon_tmp;
 					Xdialog.check = check_tmp;
@@ -1083,6 +1263,7 @@ show_again:
 			Xdialog.reverse		= FALSE;
 			Xdialog.keep_colors	= FALSE;
 			Xdialog.interval	= 0;
+			Xdialog.timeout		= 0;
 			Xdialog.icon		= FALSE;
 			Xdialog.buttons		= TRUE;
 			Xdialog.ok_button	= TRUE;
